import SimpleOpenNI.*;  // simple-openni

SimpleOpenNI kinect;

void setup() {
  kinect = new SimpleOpenNI(this);
  kinect.setMirror(true);                            // ミラー反転を有効化
  kinect.enableDepth();                              // 深度画像を有効化
  kinect.enableRGB();                                // カラー画像を有効化
  kinect.enableUser(SimpleOpenNI.SKEL_PROFILE_ALL);  // ユーザトラッキングを有効化
  kinect.alternativeViewPointDepthToImage();         // 視点の位置合わせ
  size(kinect.rgbWidth(), kinect.rgbHeight());       // 画面サイズの設定
}

void draw() {
  kinect.update();                 // Kinectのデータの更新
  image(kinect.rgbImage(), 0, 0);  // カラー画像の描画

  //ユーザごとに骨格のトラッキングができていたら骨格を描画
  for (int userId = 1; userId <= kinect.getNumberOfUsers(); userId++) {
    if( kinect.isTrackingSkeleton(userId) ) {
      strokeWeight(10);       // 線の太さの設定
      stroke(255,0,0);        // 線の色の設定
      drawSkeleton(userId);   // 骨格の描画
      detectGesture(userId);  // ジェスチャ認識
    }
  }
}

// 新しいユーザを見つけた場合の処理
void onNewUser(int userId) {
  kinect.requestCalibrationSkeleton(userId, true);  // 骨格キャリブレーションのリクエスト
}

// 骨格キャリブレーション終了時の処理
void onEndCalibration(int userId, boolean successfull) {
  if (successfull) { 
    kinect.startTrackingSkeleton(userId);  // 骨格トラッキングの開始
  }
}

// 骨格の描画
void drawSkeleton(int userId) {
  // 関節間を結ぶ直線を描画
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_HEAD,           SimpleOpenNI.SKEL_NECK);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_NECK,           SimpleOpenNI.SKEL_LEFT_SHOULDER);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_SHOULDER,  SimpleOpenNI.SKEL_LEFT_ELBOW); 
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_ELBOW,     SimpleOpenNI.SKEL_LEFT_HAND);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_NECK,           SimpleOpenNI.SKEL_RIGHT_SHOULDER);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_SHOULDER, SimpleOpenNI.SKEL_RIGHT_ELBOW);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_ELBOW,    SimpleOpenNI.SKEL_RIGHT_HAND);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_SHOULDER,  SimpleOpenNI.SKEL_TORSO);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_SHOULDER, SimpleOpenNI.SKEL_TORSO);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_TORSO,          SimpleOpenNI.SKEL_LEFT_HIP);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_HIP,       SimpleOpenNI.SKEL_LEFT_KNEE);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_KNEE,      SimpleOpenNI.SKEL_LEFT_FOOT);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_TORSO,          SimpleOpenNI.SKEL_RIGHT_HIP);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_HIP,      SimpleOpenNI.SKEL_RIGHT_KNEE);
  kinect.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_KNEE,     SimpleOpenNI.SKEL_RIGHT_FOOT);  
}

// ジェスチャ認識
void detectGesture(int userId) {
  // 右手の3次元位置を取得する
  PVector hand3d = new PVector(); // 3次元位置
  kinect.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_RIGHT_HAND, hand3d);

  // 右手の2次元位置を取得する
  PVector hand2d = new PVector(); // 2次元位置
  kinect.convertRealWorldToProjective(hand3d, hand2d);

  // 右肩の3次元位置を取得する
  PVector shoulder3d = new PVector();
  kinect.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_RIGHT_SHOULDER, shoulder3d);
  
  // 右手が右肩よりも高い位置にあるとき，手の2次元位置に円を表示する
  if ( hand3d.y - shoulder3d.y > 0 ) {
    ellipse( hand2d.x, hand2d.y, 50, 50 );
  }
}

